(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('tslib')) :
  typeof define === 'function' && define.amd ? define(['exports', 'tslib'], factory) :
  (global = global || self, factory(global.moulinette = {}, global.tslib_1));
}(this, function (exports, tslib_1) { 'use strict';

  function isMergeable(value) {
      return value !== null && typeof value === 'object' && value.constructor === Object; // prettier-ignore
  }
  function flatten(arr, res) {
      if (res === void 0) { res = []; }
      var i, cur;
      var len = arr.length;
      for (i = 0; i < len; i++) {
          cur = arr[i];
          Array.isArray(cur) ? flatten(cur, res) : res.push(cur);
      }
      return res;
  }
  function merge(target, source) {
      if (!isMergeable(target) || !isMergeable(source))
          return source;
      var merged = tslib_1.__assign({}, target);
      for (var key in source) {
          merged[key] = merge(target[key], source[key]);
      }
      return merged;
  }
  function apply(acc, fn) {
      return fn(acc) || acc;
  }
  function compose(fns) {
      return function (input) { return fns.reduceRight(apply, input); };
  }
  function moulinettify(declaration) {
      return isMergeable(declaration)
          ? function (props) { return merge(declaration, props); }
          : declaration;
  }

  function createSystem(builder, moulinettes, extensions) {
      if (moulinettes === void 0) { moulinettes = []; }
      if (extensions === void 0) { extensions = []; }
      var moulinette = compose(moulinettes);
      var System = builder(moulinette);
      System.with = function () {
          var extras = [];
          for (var _i = 0; _i < arguments.length; _i++) {
              extras[_i] = arguments[_i];
          }
          return createSystem(builder, moulinettes.concat(flatten(extras).map(moulinettify)), extensions);
      }; // prettier-ignore
      System.extend = function (extension) {
          return createSystem(builder, moulinettes, extensions.concat([extension]));
      };
      extensions.forEach(function (extend) { return extend(System); });
      return System;
  }

  exports.createSystem = createSystem;
  exports.compose = compose;
  exports.merge = merge;
  exports.default = createSystem;

  Object.defineProperty(exports, '__esModule', { value: true });

}));
